from kooki.steps import Content, Extension, Metadata, Document, Xelatex, Template
from kooki.utensils import MarkdownToHTML, HTMLToTeX, Empy, FrontMatter

from kooki.tools import write_file

def recipe(document, config):
    extensions = Extension(config, filters=['.tex'])
    metadata = Metadata(config, document.metadata)

    front_matter = FrontMatter()
    template = Empy({**extensions, **metadata})
    html = MarkdownToHTML()
    tex = HTMLToTeX()
    utensils = [front_matter, template, html, tex]
    content = Content(config, document.content, utensils)

    document_content = Document(config, document.template, extensions, {**extensions, **metadata, 'content': content})
    name = Template(document.name, metadata)

    write_file('name.tex', document_content)
    Xelatex(config, name, document_content)



from kooki.tools import get_extension, read_file, write_file
from kooki.document_exception import DocumentException

import yaml

class Kooki():
    def __init__(self, source):

        try:
            self._content = read_file(source)
            self._extension = get_extension(source)
            self.utensils = []

        except FileNotFoundError as e:
            raise DocumentException('No such file: {0}'.format(source))

    def __call__(self, **kwargs):
        result = self.cook(**{'metadata': kwargs})
        return result['content']

    def cook(self, **kwargs):

        result = {'content': self._content}
        result.update(kwargs)

        for utensil in self.utensils:
            utensil_result = utensil(**result)
            result.update(utensil_result)

        return result

    def content(self):
        return self._content

    def extension(self):
        return self._extension

    def add_utensil(self, utensil):
        self.utensils.append(utensil)

    def remove_utensil(self):
        self.utensils.pop()


class ContentKooki(Kooki):

    def __init__(self, content):
        self._content = content
        self.utensils = []



from kooki.tools.output import Output
from kooki.tools import read_file
from kooki import DocumentException

def Read(files):
    Output.start_step('read')
    result = ''
    infos = []
    error = False
    for source in files:
        try:
            result += read_file(source)
            infos.append({'name': source, 'status': '[found]', 'info': ''})

        except DocumentException as e:
            infos.append({'name': source, 'status': ('[error]', 'red'), 'info': (str(e), 'red')})
            error = True

        except NameError as e:
            infos.append({'name': source, 'status': ('[error]', 'red'), 'info': (str(e), 'red')})
            error = True

        except Exception as e:
            raise Exception(type(e))

    if len(files) > 0:
        Output.infos(infos, [('name', 'cyan'), ('status', 'green'), ('info', 'cyan')])

    if error:
        raise DocumentException('Something went wrong... :(')

    return result



from kooki.kooki import ContentKooki
from kooki.tools.output import Output
from kooki import DocumentException

def Custom(parts, merger):
    Output.start_step('custom')
    result = []
    for index, part in enumerate(parts):
        result.append(merger(part, index))
    return result



from kooki import DocumentException
from kooki.tools.output import Output

def Document(config, template, extensions, metadata):

    Output.start_step('document')

    if template in extensions:
        kooki = extensions[template]
        infos = []
        infos.append({'name': template, 'status': '[found]', 'info': ''})
        Output.infos(infos, [('name', 'blue'), ('status', 'green'), ('info', 'cyan')])
    else:
        raise DocumentException('the template provided (\'{}\') was not found'.format(template))

    document = kooki(**metadata)

    return document



from kooki.steps import Extension, Metadata, Document, Content, Sass, Font, Css, Js, Template, Export
from kooki.utensils import MarkdownToHTML, HTMLToTeX, Empy, FrontMatter

def recipe(document, config):
    extensions = Extension(config, filters=['.html'])
    metadata = Metadata(config, document.metadata)
    fonts = Font(config)
    js = Js(config)
    css = Css(config)
    sass = Sass(config)

    front_matter = FrontMatter()
    template = Empy({**extensions, **metadata})
    html = MarkdownToHTML()
    utensils = [front_matter, template, html]
    content = Content(config, document.content, utensils)

    document_content = Document(config, document.template, extensions, metadata={
        **extensions,
        **metadata,
        'js': js,
        'css': css,
        'sass': sass,
        'fonts': fonts,
        'content': content})

    name = Template(document.name, metadata)

    Export(name, document_content, 'html')



import os, yaml, logging
import kooki
from kooki.tools import DictAsMember
from kooki import DocumentException

import pykwalify
import pykwalify.core
import pkgutil

class EmptyLog():
    def debug(*args, **kwargs):
        pass
    def error(*args, **kwargs):
        pass
    def info(*args, **kwargs):
        pass

def parse_args(args):
    config_file_checker(args.config_file)
    config = config_file_parser(args.config_file)
    document_rules = create_document_rules(config, args.documents)
    return document_rules


def config_file_checker(config_file):

    if not os.path.isfile(config_file):
        raise DocumentException('The config file provided (\'{}\') does not exist'.format(config_file))

    format_path = os.path.join(os.path.dirname(kooki.config.__file__), 'format.yaml')
    pykwalify.core.log = EmptyLog()
    c = pykwalify.core.Core(source_file=config_file, schema_files=[format_path])

    try:
        c.validate(raise_exception=True)
        return c.source

    except pykwalify.errors.SchemaError as e:
        message = 'The config file provided (\'{}\') is invalid\n'.format(config_file)
        for error in c.errors[:-1]:
            message += '{}\n'.format(error)
        message += '{}'.format(c.errors[-1])
        raise DocumentException(message)


def config_file_parser(config_file):
    with open(config_file, 'r') as stream:
        config = yaml.safe_load(stream.read())
        return config


def create_document_rules(config, documents):

    default = {
        'name': '',
        'template': '',
        'recipe': '',
        'jars': [],
        'metadata': [],
        'content': []}

    document_names = []

    for key, value in config.items():
        if key in default:
            default[key] = value
        else:
            document_names.append(key)

    document_rules = {}

    if documents == []:
        for document_name in document_names:
            specific = default.copy()
            for key, value in config[document_name].items():
                if key in specific:
                    specific[key] = value
            document_rules[document_name] = DictAsMember(specific)
    else:
        for document_name in documents:
            if document_name in document_names:
                specific = default.copy()
                for key, value in config[document_name].items():
                    if key in specific:
                        specific[key] = value
                document_rules[document_name] = DictAsMember(specific)
            else:
                raise DocumentException('There is no rule for the document \'{}\''.format(document_name))

    return DictAsMember(document_rules)



#!/usr/bin/env python
import os, sys
from setuptools import setup, find_packages

__version__ = '0.4.10'

setup(
    name='kooki',
    version=__version__,
    description='The ultimate document generator.',
    author='Noel Martignoni',
    include_package_data=True,
    package_data={'kooki.config': ['format.yaml']},
    author_email='noel@martignoni.fr',
    url='https://gitlab.com/kooki/kooki',
    scripts=['scripts/kooki'],
    install_requires=['markdown', 'empy', 'pyyaml', 'toml', 'requests', 'termcolor', 'vcstool', 'libsass', 'pykwalify'],
    packages=find_packages(exclude=['tests*', 'jars']),
    test_suite='tests',
)



from kooki.steps import Content, Extension, Metadata, Document, Template, Export
from kooki.utensils import Empy, FrontMatter

def recipe(document, config):
    extensions = Extension(config, filters=['.md'])
    metadata = Metadata(config, document.metadata)

    front_matter = FrontMatter()
    template = Empy({**extensions, **metadata})
    utensils = [front_matter, template]
    content = Content(config, document.content, utensils)

    document_content = Document(config, document.template, extensions, {**extensions, **metadata, 'content': content})
    name = Template(document.name, metadata)

    Export(name, document_content, 'md')
